# -*- coding: utf-8 -*-

from django.forms import widgets
from django.utils.safestring import mark_safe
from django.db.models.fields.files import FieldFile
from django.template.loader import render_to_string
from djangoplus.ui.components.select import widgets as select_widgets
from djangoplus.ui.components.editor import widgets as editor_widgets
from djangoplus.ui.components.calendar import widgets as calendar_widgets
from djangoplus.utils.metadata import get_metadata, get_fiendly_name

# Date Widgets
DateFilterWidget = calendar_widgets.DateFilterWidget
DateTimeWidget = calendar_widgets.DateTimeWidget
DateRangeWidget = calendar_widgets.DateRangeWidget
DateWidget = calendar_widgets.DateWidget
HiddenDateWidget = calendar_widgets.HiddenDateWidget

# Richtext Widget
FormattedTextarea = editor_widgets.FormattedTextarea

# Select Widgets
SelectWidget = select_widgets.SelectWidget
SelectMultipleWidget = select_widgets.SelectMultipleWidget
NullBooleanSelect = select_widgets.NullBooleanSelect

# Base Widgets #
from djangoplus.utils import format_value


class TextInput(widgets.TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs.update(**{'class':'form-control {}'.format((attrs or self.attrs).get('class', ''))})
        return super(TextInput, self).render(name, value, attrs, renderer)


class EmailInput(TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs['type'] = 'email'
        html = super(EmailInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-at"></i></span>
        </div>
        '''.format(html)
        return html


class UrlInput(TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs['type'] = 'url'
        html = super(UrlInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-link"></i></span>
        </div>
        '''.format(html)
        return html


class AddressInput(TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        html = super(AddressInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-envelope-o"></i></span>
        </div>
        '''.format(html)
        return html


class MaskWidget(widgets.TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def __init__(self, mask=''):
        super(MaskWidget, self).__init__()
        self.mask = mask

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(MaskWidget, self).render(name, value, attrs)
        script = '<script>$("#id_{}").mask("{}", {{clearIfNotMatch: true}});</script>'.format(name, self.mask)
        return mark_safe('{}\n\n{}'.format(html, script))


class HiddenInput(widgets.HiddenInput):
    pass


class ReadOnlyInput(TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        self.attrs['readonly'] = 'readonly'
        return super(ReadOnlyInput, self).render(name, value, attrs)


class DisplayInput(widgets.TextInput):
    def __init__(self, obj=None):
        super(DisplayInput, self).__init__()
        self.obj = obj

    def render(self, name, value, attrs=None, renderer=None):
        if value:
            if isinstance(value, FieldFile):
                value = value and str(value) or value.field.default
                url = '/static/' in value and value or '/media/{}'.format(value)
                height = value.split('.')[-1] in ('pdf', 'txt', 'xls', 'docx') and 500 or ''
                return mark_safe('<embed src="{}" width="100%" height="{}">'.format(url, height))
            return '<br>{}'.format(format_value(value))
        elif self.obj and hasattr(self.obj, 'pk'):
                return '''
                    <input class="form-control" type="text" value="{}" disabled />
                    <input type="hidden" id="id_{}" name="{}" value="{}"/>
                '''.format(self.obj, name, name, self.obj.pk)
        else:
            return ''


class Textarea(widgets.Textarea):
    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        if 'rows' not in attrs:
            attrs['rows'] = 3
        html = super(Textarea, self).render(name, value, attrs)
        return mark_safe(html)


class NumberInput(widgets.NumberInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(NumberInput, self).render(name, value, attrs)
        if self.max_value is not None and self.min_value is not None:
            html = html  # TODO range widget
        return mark_safe(html)


class DecimalInput(widgets.TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(DecimalInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('###.###.##0,00', {{reverse: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class MoneyInput(DecimalInput):
    def render(self, name, value, attrs=None, renderer=None):
        html = super(MoneyInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon">R$</span>
        </div>
        '''.format(html)
        return mark_safe(html)


class CheckboxInput(widgets.CheckboxInput):
    def render(self, *args, **kwargs):
        html = super(CheckboxInput, self).render(*args, **kwargs)
        html = '{}<span class="custom-checkbox"></span>'.format(html)
        return mark_safe(html)


class RadioSelect(widgets.RadioSelect):
    input_type = 'radio'

    def render(self, *args, **kwargs):
        html = super(RadioSelect, self).render(*args, **kwargs)
        html = '{}'.format(html)
        lines = []
        for line in html.split('\n'):
            if '<li><label ' in line:
                line = '{}<span class="custom-radio"></span>'.format(line)
            lines.append(line)
        return mark_safe('\n'.join(lines))


class PickWidget(widgets.Select):
    def __init__(self, template_name='pick_widget.html', multiple=False, grouper=None, *args, **kwargs):
        super(PickWidget, self).__init__(*args, **kwargs)
        self.template_name = template_name
        self.allow_multiple_selected = multiple
        self.grouper = grouper

    def value_from_datadict(self, data, files, name):
        if self.allow_multiple_selected:
            getter = data.getlist
        else:
            getter = data.get
        return getter(name)

    def render(self, name, value, attrs=None, renderer=None, choices=()):
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs)
        values = value and (type(value) == int and [value] or [int(v) for v in value]) or []
        widget_cls = self.allow_multiple_selected and CheckboxInput or RadioSelect
        i = 0
        grouped_objects = []
        onclick = "var is=this.parentNode.parentNode.parentNode.parentNode.parentNode.getElementsByTagName('input');" \
            "for(var i=0; i<is.length; i++) is[i].checked = {}".format(
                self.allow_multiple_selected and 'this.checked' or 'false')
        widget = widget_cls({'onclick': onclick}, check_test=lambda v: False).render(None, '')
        extra_display = []
        if self.choices:
            qs = hasattr(self.choices.queryset, 'all') and self.choices.queryset.all() or self.choices.queryset
            select_display = get_metadata(qs.model, 'select_display')
            if select_display:
                for lookup in get_metadata(qs.model, 'select_display', []):
                    extra_display.append((get_fiendly_name(qs.model, lookup, as_tuple=False), lookup))
            else:
                extra_display.append((None, '__str__'))
            if self.grouper:
                groupers = qs.values_list(self.grouper, flat=True).order_by(self.grouper).distinct()
            else:
                groupers = [None]
            for grouper in groupers:
                objects = []
                if grouper:
                    grouped_qs = qs.filter(**{self.grouper : grouper})
                else:
                    grouped_qs = qs.all()
                for obj in grouped_qs:
                    option_value = obj.pk
                    if has_id:
                        final_attrs = dict(final_attrs, id='{}_{}'.format(attrs['id'], i))
                    final_attrs['class'] = self.allow_multiple_selected and 'custom-checkbox' or 'custom-radio'
                    obj.widget = widget_cls(final_attrs, check_test=lambda v: int(v) in values).render(
                        name, str(option_value)
                    )
                    i += 1
                    objects.append(obj)
                grouped_objects.append((grouper, objects))

        return mark_safe(
            render_to_string(
                self.template_name, dict(grouped_objects=grouped_objects, widget=widget, extra_display=extra_display, name=name.replace('-', '_'))
            )
        )


# File Widgets #


class FileInput(widgets.ClearableFileInput):
    template_name = 'clearable_file_input.html'
    pass


class ImageInput(widgets.ClearableFileInput):
    template_name = 'clearable_file_input.html'
    pass


class PhotoWidget(widgets.TextInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs.update(style='display:none;')
        html = super(PhotoWidget, self).render(name, value, attrs)
        return render_to_string('widgets/photo_widget.html', dict(name=name, html=html))


# Regional Widgets #

class CpfWidget(MaskWidget):
    def __init__(self):
        super(CpfWidget, self).__init__('000.000.000-00')


class CnpjWidget(MaskWidget):
    def __init__(self):
        super(CnpjWidget, self).__init__('00.000.000/0000-00')


class CpfCnpjWidget(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value=None, attrs=None, renderer=None):
        html = super(CpfCnpjWidget, self).render(name, value, attrs)
        html = '''
                <div class="input-group">
                    {}
                </div>
                <script>
                    var CpfCnpjMaskBehavior = function (val) {{
                        return val.replace(/\D/g, '').length <= 11 ? '000.000.000-009' : '00.000.000/0000-00';
                    }},
                    spOptions = {{
                      clearIfNotMatch: true,
                      onKeyPress: function(val, e, field, options) {{
                          field.mask(CpfCnpjMaskBehavior.apply({}, arguments), options);
                        }}
                    }};

                    $('#id_{}').mask(CpfCnpjMaskBehavior, spOptions);
                </script>
                '''.format(html, value, name)
        return mark_safe(html)


class CepWidget(MaskWidget):
    def __init__(self):
        super(CepWidget, self).__init__('00.000-000')


class CarPlateWidget(MaskWidget):
    def __init__(self):
        super(CarPlateWidget, self).__init__('AAA-0000')


class MercosulCarPlateWidget(MaskWidget):
    def __init__(self):
        super(MercosulCarPlateWidget, self).__init__('AAA-0A00')


class PhoneWidget(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs=None, renderer=None):
        html = super(PhoneWidget, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-phone"></i></span>
        </div>
        <script>
            var SPMaskBehavior = function (val) {{
              return val.replace(/\D/g, '').length === 11 ? '(00) 00000-0000' : '(00) 0000-00009';
            }},
            spOptions = {{
              clearIfNotMatch: true,
              onKeyPress: function(val, e, field, options) {{
                  field.mask(SPMaskBehavior.apply({{}}, arguments), options);
                }}
            }};

            $('#id_{}').mask(SPMaskBehavior, spOptions);
        </script>
        '''.format(html, name)
        return mark_safe(html)

# Utilitary Fields #


class PasswordInput(widgets.PasswordInput):
    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        return super(PasswordInput, self).render(name, value, attrs)


class DecimalInput3(DecimalInput):

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(DecimalInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('#.##0,000', {{reverse: true, clearIfNotMatch: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class DecimalInput1(DecimalInput):

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(DecimalInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('#0,0', {{reverse: true, clearIfNotMatch: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class OneDigitValidationInput(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'form-control'
        html = super(OneDigitValidationInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('#.##0-0', {{reverse: true, clearIfNotMatch: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class CreditCardWidget(MaskWidget):
    def __init__(self):
        super(CreditCardWidget, self).__init__('9999 9999 9999 9999')


class ColorInput(TextInput):

    class Media:
        js = ('/static/js/colorPick.min.js',)
        css = {'all': ('/static/css/colorPick.min.css',)}

    def render(self, name, value, attrs=None, renderer=None):
        attrs['class'] = 'hidden'
        html = super(ColorInput, self).render(name, value, attrs)
        cpid = '{}ColorPickSelector'.format(name)
        js = '''
            <div id="{}ColorPickSelector" class="colorPickSelector" style="cursor:pointer"></div>
            <script>
            $(function(){{
            $("#{}ColorPickSelector").colorPick({{
                'initialColor' : '{}',
                'onColorSelected': function() {{
                     $("#id_{}").val(this.color);
                    this.element.css({{'backgroundColor': this.color, 'color': this.color}});
                }}
            }});
            }});
            </script>
        '''.format(cpid, cpid, value or '#27ae60', name)
        return mark_safe('{}\n{}'.format(html, js))



# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand
from djangoplus.utils.storage.dropbox import DropboxStorage


class Command(BaseCommand):
    def handle(self, *args, **options):
        DropboxStorage().sync()



# -*- coding: utf-8 -*-
import re
from django.conf import settings
from djangoplus.test import TestCase
from djangoplus.admin.models import User
from djangoplus.mail.utils import load_emails
from djangoplus.test.decorators import testcase
from django.utils.translation import ugettext as _


class AdminTestCase(TestCase):

    def test(self):
        User.objects.create_superuser(settings.DEFAULT_SUPERUSER, None, settings.DEFAULT_PASSWORD)
        self.execute_flow()

    @testcase('Configure')
    def configure(self):
        self.click_icon(_('Settings'))
        self.click_link(_('Edit Profile'))
        self.enter(_('Name'), 'Administrador')
        self.enter(_('Password'), '123')
        self.enter(_('Confirm Password'), '123')
        self.click_button(_('Save'))
        self.logout()
        self.login('admin', '123')
        self.click_icon(_('Settings'))
        self.click_link(_('Settings'))
        self.enter(_('Name'), 'My Project')
        self.click_button(_('Save'))

    @testcase('Create User')
    def create_user(self):
        self.click_icon(_('Users'))
        self.click_link(_('Add'))
        self.enter(_('Name'), 'Carlos Breno')
        self.enter(_('E-mail'), 'brenokcc@yahoo.com.br')
        self.enter(_('Username'), 'brenokcc')
        self.enter(_('Password'), 'senha')
        self.click_button(_('Save'))

    @testcase('Check User Password', username='brenokcc', password='senha')
    def check_user_password(self):
        self.look_at('Carlos Breno')

    @testcase('Check User Password')
    def check_user_password(self):
        self.click_icon(_('Users'))
        self.look_at('Carlos Breno')
        self.click_button(_('Change Password'))
        self.enter(_('Password'), '321')
        self.enter(_('Confirm Password'), '321')
        self.click_button(_('Save'))

    @testcase('Check User Password', username='brenokcc', password='321')
    def check_user_password_again(self):
        self.look_at('Carlos Breno')

    @testcase('Reset Password', username=None, password=None)
    def reset_password(self):
        self.open('/admin/logout/')
        self.open('/admin/login/')
        self.click_link(_('Reset Password'))
        self.enter('E-mail', 'brenokcc@yahoo.com.br')
        # settings.EMAIL_BACKEND = 'djangoplus.mail.backends.EmailDebugBackend'
        self.click_button(_('Send E-mail'))
        emails = load_emails()
        url = re.findall('/admin/password/.*/"', '\n'.join(emails[0]['alternatives'][0]))[0][0:-1]
        self.open(url)
        self.enter(_('Password'), '111')
        self.enter(_('Confirm Password'), '111')
        self.click_button(_('Save'))
        self.logout()



# Generated by Django 2.0.8 on 2019-02-12 15:39

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0008_auto_20190212_1519'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='group',
            options={'verbose_name': 'Group', 'verbose_name_plural': 'Groups'},
        ),
        migrations.AlterModelOptions(
            name='organization',
            options={'verbose_name': 'Organization', 'verbose_name_plural': 'Organizations'},
        ),
        migrations.AlterModelOptions(
            name='permission',
            options={'verbose_name': 'Permission', 'verbose_name_plural': 'Permissions'},
        ),
        migrations.AlterModelOptions(
            name='role',
            options={'verbose_name': 'Role', 'verbose_name_plural': 'Roles'},
        ),
        migrations.AlterModelOptions(
            name='settings',
            options={'verbose_name': 'Settings', 'verbose_name_plural': 'Settings'},
        ),
        migrations.AlterModelOptions(
            name='unit',
            options={'verbose_name': 'Unit', 'verbose_name_plural': 'Units'},
        ),
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'User', 'verbose_name_plural': 'Users'},
        ),
        migrations.AlterField(
            model_name='log',
            name='content',
            field=djangoplus.db.models.fields.TextField(null=True, verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='log',
            name='content_type',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.ContentType', verbose_name='Content Type'),
        ),
        migrations.AlterField(
            model_name='log',
            name='date',
            field=djangoplus.db.models.fields.DateTimeField(auto_now=True, verbose_name='Date/Time'),
        ),
        migrations.AlterField(
            model_name='log',
            name='object_description',
            field=djangoplus.db.models.fields.CharField(max_length=255, verbose_name='Object Description'),
        ),
        migrations.AlterField(
            model_name='log',
            name='object_id',
            field=djangoplus.db.models.fields.IntegerField(verbose_name='Identifier'),
        ),
        migrations.AlterField(
            model_name='log',
            name='operation',
            field=djangoplus.db.models.fields.IntegerField(choices=[[1, 'Add'], [2, 'Edit'], [3, 'Delete']], verbose_name='Operation'),
        ),
        migrations.AlterField(
            model_name='log',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='logindex',
            name='content_type',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Content Type'),
        ),
        migrations.AlterField(
            model_name='logindex',
            name='object_id',
            field=djangoplus.db.models.fields.IntegerField(verbose_name='Identifier'),
        ),
        migrations.AlterField(
            model_name='role',
            name='group',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Group', verbose_name='Group'),
        ),
        migrations.AlterField(
            model_name='role',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='address',
            field=djangoplus.db.models.fields.TextField(blank=True, null=True, verbose_name='Address'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='company',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Company Name'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='email',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Email'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='icon',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='blank.png', null=True, upload_to='config', verbose_name='Icon'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='initials',
            field=djangoplus.db.models.fields.CharField(default='Django+', max_length=255, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='logo',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='', null=True, upload_to='config', verbose_name='Logo'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='logo_pdf',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='', help_text='No-background image', null=True, upload_to='config', verbose_name='PDF Logo'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='name',
            field=djangoplus.db.models.fields.CharField(default='Django Plus', max_length=255, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='phone_1',
            field=djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Primary Phome'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='phone_2',
            field=djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Secondary Phone'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='pinterest',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Pinterest'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='server_address',
            field=djangoplus.db.models.fields.CharField(default='http://localhost:8000', max_length=255, verbose_name='Server URL'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='system_email_address',
            field=djangoplus.db.models.fields.CharField(default='no-reply@djangoplus.net', max_length=255, verbose_name='Notificaton E-mail'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='version',
            field=djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='System Version'),
        ),
        migrations.AlterField(
            model_name='user',
            name='active',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.AlterField(
            model_name='user',
            name='name',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=30, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.JsonField(null=True, verbose_name='Permissions Mapping'),
        ),
        migrations.AlterField(
            model_name='user',
            name='photo',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='user.png', null=True, upload_to='profiles', verbose_name='Photo'),
        ),
        migrations.AlterField(
            model_name='user',
            name='username',
            field=djangoplus.db.models.fields.CharField(max_length=30, unique=True, verbose_name='Username'),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import os
import sys
import json
import uuid
import binascii
import operator
from functools import reduce
from django.db.models import Q
from django.core import signing
from djangoplus.db import models
from django.conf import settings
from djangoplus.mail import send_mail
from django.utils.translation import ugettext as _
# from django.utils.translation import gettext_lazy as _
from djangoplus.decorators import action, meta, subset
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.models import ContentTypeManager
from djangoplus.utils.metadata import get_metadata, getattr2, get_field
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, \
    UserManager as DjangoUserManager, Group, Permission


class ContentTypeManager(ContentTypeManager):
    pass


class ContentType(ContentType):
    class Meta:
        proxy = True

    objects = ContentTypeManager()


class Log(models.Model):
    ADD = 1
    EDIT = 2
    DELETE = 3

    OPERATION_CHOICES = [[ADD, _('Add')], [EDIT, _('Edit')], [DELETE, _('Delete')]]

    content_type = models.ForeignKey(ContentType, verbose_name=_('Content Type'), filter=True)
    operation = models.IntegerField(verbose_name=_('Operation'), choices=OPERATION_CHOICES, filter=True)
    user = models.ForeignKey('admin.User', filter=True, verbose_name=_('User'))
    date = models.DateTimeField(verbose_name=_('Date/Time'), auto_now=True, filter=True)
    object_id = models.IntegerField(verbose_name=_('Identifier'), search=True)
    object_description = models.CharField(verbose_name=_('Object Description'))
    content = models.TextField(verbose_name=_('Content'), null=True, search=True)

    fieldsets = (
        (_('General Data'), {'fields': (
            ('content_type', 'operation'), ('user', 'date'), ('object_id', 'object_description'), 'get_tags')}),
        (_('Indexes'), {'relations': ('logindex_set',), 'condition': 'has_index'}),
    )

    objects = models.Manager()

    class Meta:
        verbose_name = _('Log')
        verbose_name_plural = _('Logs')
        icon = 'fa-history'
        list_per_page = 25
        list_display = 'content_type', 'object_id', 'operation', 'user', 'date', 'get_tags'
        order_by = '-date'

    def __str__(self):
        return 'Log #{}'.format(self.pk)

    def can_add(self):
        return False

    def can_edit(self):
        return False

    def can_delete(self):
        return False

    def has_index(self):
        return self.logindex_set.exists()

    def get_action_description(self):
        return (_('added'), _('edited'), _('deleted'))[self.operation - 1]

    def get_style(self):
        return ('success', 'info', 'danger')[self.operation - 1]

    def get_icon(self):
        return ('plus', 'pencil', 'trash-o')[self.operation - 1]

    @meta('Alterações')
    def get_tags(self):
        data = []
        for attr, old, new in json.loads(self.content):
            data.append('{} : {} >> {}'.format(attr, old, new))

        for log_index in self.logindex_set.all():
            date = datetime.datetime(
                log_index.log.date.year, log_index.log.date.month, log_index.log.date.day,
                log_index.log.date.hour, log_index.log.date.minute, log_index.log.date.second
            )
            qs = Log.objects.filter(
                content_type=log_index.content_type, user=log_index.log.user, date__startswith=date,
                object_id=log_index.object_id
            )
            for log in qs:
                for attr, old, new in json.loads(log.content):
                    data.append('{} : {} >> {}'.format(attr, old, new))

        return ' | '.join(data)

    def create_indexes(self, instance):
        for log_index in get_metadata(instance.__class__, 'logging', (), iterable=True):
            index_object = getattr2(instance, log_index)
            if index_object:
                index_content_type = ContentType.objects.get_for_model(index_object.__class__)
                LogIndex.objects.create(log=self, content_type=index_content_type, object_id=index_object.pk)


class LogIndex(models.Model):
    log = models.ForeignKey(Log, verbose_name=_('Log'), composition=True)
    content_type = models.ForeignKey('contenttypes.ContentType', verbose_name=_('Content Type'))
    object_id = models.IntegerField(verbose_name=_('Identifier'), search=True)

    class Meta:
        verbose_name = _('Index')
        verbose_name_plural = _('Indexes')
        list_display = 'content_type', 'object_id', 'get_tags'

    def __str__(self):
        return 'Index #{}'.format(self.pk)

    @meta('Alterações')
    def get_tags(self):
        data = []
        date = datetime.datetime(self.log.date.year, self.log.date.month, self.log.date.day, self.log.date.hour,
                                 self.log.date.minute, self.log.date.second)
        qs = Log.objects.filter(content_type=self.content_type, user=self.log.user, date__startswith=date,
                                object_id=self.object_id)
        for log in qs:
            for attr, old, new in json.loads(log.content):
                data.append('{} : {} >> {}'.format(attr, old, new))
        return ' | '.join(data)


class Scope(models.AsciiModel):
    class Meta:
        verbose_name = _('Scope')
        verbose_name_plural = _('Scopes')

    def is_organization(self):
        return Organization.objects.filter(pk=self.pk).first()

    def is_unit(self):
        return Unit.objects.filter(pk=self.pk).first()


class Organization(Scope):
    class Meta:
        verbose_name = _('Organization')
        verbose_name_plural = _('Organizations')

    @classmethod
    def subclass(cls):
        return cls.__subclasses__() and cls.__subclasses__()[0] or None

    def get_units(self):
        unit_subclass = Unit.subclass()
        if unit_subclass:
            for field in get_metadata(unit_subclass, 'fields'):
                if field.remote_field and hasattr(field.remote_field.model, 'organization_ptr'):
                    return unit_subclass.objects.filter(**{field.name:self.pk})
        return Unit.objects.none()


class Unit(Scope):
    class Meta:
        verbose_name = _('Unit')
        verbose_name_plural = _('Units')

    @classmethod
    def subclass(cls):
        return cls.__subclasses__() and cls.__subclasses__()[0] or None

    def get_organization(self):
        organization_subclass = Organization.subclass()
        if organization_subclass:
            for field in get_metadata(type(self), 'fields'):
                if field.remote_field and field.remote_field.model == organization_subclass:
                    return getattr(self, field.name)
        return None

    @classmethod
    def get_organization_field_name(cls):
        organization_subclass = Organization.subclass()
        unit_subclass = cls.subclass()
        if organization_subclass and unit_subclass:
            for field in get_metadata(unit_subclass, 'fields'):
                if field.remote_field and field.remote_field.model == organization_subclass:
                    return field.name
        return None


class UserQuerySet(models.QuerySet):

    @subset('Ativos')
    def active(self):
        return self.filter(active=True)

    @subset('Inativos')
    def inactive(self):
        return self.filter(active=False)

    @classmethod
    @action('Token', input='LoginForm')
    def get_token(cls, username, password):
        user = User.objects.get(username=username)
        return user.token


class UserManager(DjangoUserManager):

    def all(self, *args, **kwargs):
        return super(UserManager, self).all()

    def get_queryset(self):
        return UserQuerySet(self.model, using=self._db)

    def create_user(self, username, email, password=None, is_superuser=False):
        if not username:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        user = self.model(username=username, email=email, active=True, is_superuser=is_superuser, last_login=None)
        if password:
            user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username, email, password):
        return self.create_user(username, email, password, True)


class User(AbstractBaseUser, PermissionsMixin):
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    name = models.CharField(_('Name'), max_length=100, blank=True, search=True)
    username = models.CharField(_('Username'), max_length=30, unique=True, search=True)
    email = models.CharField(_('E-mail'), max_length=75, blank=True, default='')
    active = models.BooleanField(verbose_name=_('Active'), default=True, filter=True)
    photo = models.ImageField(upload_to='profiles', null=True, blank=True,
                              default='/static/images/user.png', verbose_name=_('Photo'), exclude=True)

    token = models.CharField(verbose_name='Token', null=True, exclude=True)

    permission_mapping = models.JsonField(verbose_name=_('Permissions Mapping'), exclude=True, display=False)
    scope = models.ForeignKey(Scope, verbose_name='Scope', null=True, exclude=True, blank=True)
    objects = UserManager()

    fieldsets = (
        (_('Identification'), {'fields': (('name', 'email'),), 'image': 'photo'}),
        (_('Access'), {'fields': (('username', 'is_superuser'), ('active',))}),
        (_('Roles'), {'relations': ('role_set',)}),
        (_('Permissions Mapping'), {'fields': ('permission_mapping',)}),
    )

    class Meta:
        verbose_name = _('User')
        verbose_name_plural = _('Users')
        list_display = 'photo', 'username', 'name', 'email', 'groups'
        add_form = 'UserForm'
        can_admin = _('User Manager')
        icon = 'fa-users'
        expose = True
        # list_template = 'image_cards.html'

    def __init__(self, *args, **kwargs):
        super(User, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        if not self.token:
            self.token = binascii.hexlify(os.urandom(20)).decode()
        if not self.password:
            if settings.DEBUG or 'test' in sys.argv or True:
                password = settings.DEFAULT_PASSWORD
            else:
                password = uuid.uuid4().hex
            self.set_password(password)
        super(User, self).save(*args, **kwargs)
        if self.is_superuser:
            group = Group.objects.get_or_create(name=_('Superuser'))[0]
            self.groups.add(group)
        group = Group.objects.get_or_create(name=_('User'))[0]
        self.groups.add(group)

    def __str__(self):
        return self.name or self.username

    @action(_('Change Password'), input='ChangePasswordForm', inline=True)
    def change_password(self, new_password, confirm_password):
        self.set_password(new_password)
        self.save()

    @action(_('Send Access Invitation'), inline=True, condition='email', category=None)
    def send_access_invitation(self):
        self.send_access_invitation_for_group(None)

    def send_reset_password_notification(self):
        project_name = Settings.default().initials
        subject = '{} - "{}"'.format(_('Reset Password'), project_name)
        message = _('Click on the button bellow to (re)define your password.')
        actions = [(_('Reset Password!'), '/admin/password/{}/{}/'.format(self.pk, signing.dumps(self.password)))]
        send_mail(subject, message, self.email, actions=actions)

    def send_access_invitation_for_group(self, group):
        project_name = Settings.default().initials
        subject = '{} - "{}"'.format(_('System Access'), project_name)
        if group:
            extra = _('''If you are already a user, click on the button "Access now!" to authenticate.
                Otherwise, click on the button "Define password!" to provide your access password.''')
            message = '''{} <b>{}</b> {} <b>{}</b>.
                {}
            '''.format(_('You were registered in the system.'), project_name, _(' as '), group, extra)
            actions = [
                (_('Access now!'), '/admin/'),
                (_('Define password!'), '/admin/password/{}/{}/'.format(self.pk, signing.dumps(self.password)))
            ]
        else:
            extra = _('Click on the button bellow to (re)define your password.')
            message = '''{} <b>{}</b>.
                {}.
            '''.format(_('You were registered in the system'), project_name, extra)
            actions = [(_('Access now!'), '/admin/password/{}/{}/'.format(self.pk, signing.dumps(self.password)))]
        send_mail(subject, message, self.email, actions=actions)

    def units(self, group_name=None):
        qs = self.role_set.all()
        if group_name:
            qs = qs.filter(group__name=group_name)
        return qs.values_list('scope', flat=True)

    def in_group(self, *group_names):
        return self.role_set.filter(group__name__in=group_names).exists()

    def in_other_group(self, group_name):
        return self.is_superuser or self.role_set.exclude(group__name=group_name).exists()

    def get_appliable_scopes(self):
        scopes = []
        for scope in Scope.objects.filter(pk__in=self.role_set.values_list('scope', flat=True)):
            if scope not in scopes:
                scopes.append(scope)
                if scope.is_organization():
                    for unit in scope.organization.get_units():
                        scopes.append(unit)
        return scopes

    def apply_current_scope(self, queryset):
        if self.scope:
            filters = []
            organization_lookups, unit_lookups, role_lookups = self.get_lookups(queryset.model)
            if self.scope.is_organization():
                lookups = organization_lookups
            else:
                lookups = unit_lookups
            for lookup in lookups:
                filters.append(Q(**{lookup: self.scope.pk}))
            if filters:
                queryset = queryset.filter(reduce(operator.__and__, filters))
        return queryset

    def get_lookups(self, model):
        from djangoplus.cache import CACHE

        organization_lookups = []
        unit_lookups = []
        role_lookups = dict()

        for lookup in get_metadata(model, 'list_lookups', (), iterable=True):
            field = get_field(model, lookup)
            if hasattr(field.remote_field.model, 'organization_ptr') or hasattr(field.remote_field.model, 'unit_ptr'):
                if hasattr(field.remote_field.model, 'organization_ptr'):
                    organization_lookups.append(lookup)
                if hasattr(field.remote_field.model, 'unit_ptr'):
                    unit_lookups.append(lookup)
            else:
                role_username = get_metadata(field.remote_field.model, 'role_username')
                if role_username:
                    role_lookups[get_metadata(field.remote_field.model, 'verbose_name')] = '{}__{}'.format(
                        lookup, role_username)
                for subclass in field.remote_field.model.__subclasses__():
                    role_username = get_metadata(subclass, 'role_username')
                    if role_username:
                        role_lookups[get_metadata(subclass, 'verbose_name')] = '{}__{}__{}'.format(
                            lookup, subclass.__name__.lower(), role_username)

        if hasattr(model, 'organization_ptr') and 'id' not in organization_lookups:
            organization_lookups.append('id')

        if hasattr(model, 'unit_ptr') and 'id' not in unit_lookups:
            unit_lookups.append('id')

        if get_metadata(model, 'role_username') and 'id' not in role_lookups:
            role_lookups[get_metadata(model, 'verbose_name')] = get_metadata(model, 'role_username')

        for field in get_metadata(model, 'fields') + get_metadata(model, 'many_to_many'):
            if field.remote_field and field.remote_field.model:
                if field.remote_field.model in CACHE['ROLE_MODELS']:
                    role_lookups[get_metadata(field.remote_field.model, 'verbose_name')] = '{}__{}'.format(field.name, CACHE['ROLE_MODELS'][field.remote_field.model]['username_field'])
                if field.remote_field.model in CACHE['ABSTRACT_ROLE_MODELS']:
                    for to in CACHE['ABSTRACT_ROLE_MODELS'][field.remote_field.model]:
                        role_lookups[get_metadata(to, 'verbose_name')] = '{}__{}__{}'.format(
                            field.name, to.__name__.lower(), CACHE['ROLE_MODELS'][to])
                if hasattr(field.remote_field.model, 'unit_ptr_id') and field.name not in unit_lookups:
                    unit_lookups.append(field.name)
                if hasattr(field.remote_field.model, 'organization_ptr_id') and field.name not in organization_lookups:
                    organization_lookups.append(field.name)

        for organization_lookup in organization_lookups:
            if CACHE['UNIT_MODEL']:
                for field in get_metadata(CACHE['ORGANIZATION_MODEL'], 'fields'):
                    if field.remote_field and hasattr(field.remote_field.model, 'unit_ptr'):
                        if organization_lookup == 'id':
                            unit_lookup = field.name
                        else:
                            unit_lookup = '{}__{}'.format(organization_lookup, field.name)
                        if unit_lookup not in unit_lookups and not hasattr(model, 'unit_ptr'):
                            unit_lookups.append(unit_lookup)
                        break

        for unit_lookup in unit_lookups:
            if CACHE['ORGANIZATION_MODEL']:
                for field in get_metadata(CACHE['UNIT_MODEL'], 'fields'):
                    if field.remote_field and hasattr(field.remote_field.model, 'organization_ptr'):
                        if unit_lookup == 'id':
                            organization_lookup = field.name
                        else:
                            organization_lookup = '{}__{}'.format(unit_lookup, field.name)
                        if organization_lookup not in organization_lookups and not hasattr(model, 'organization_ptr'):
                            organization_lookups.append(organization_lookup)

        return organization_lookups, unit_lookups, role_lookups

    def get_permission_mapping(self, model, obj=None):
        from djangoplus.cache import CACHE
        permission_mapping_key = obj and '{}:{}'.format(model.__name__, type(obj).__name__) or model.__name__
        if permission_mapping_key in self.permission_mapping:
            return self.permission_mapping[permission_mapping_key]

        organization_lookups, unit_lookups, role_lookups = self.get_lookups(model)
        lookups = dict(list_lookups=[], edit_lookups=[], delete_lookups=[])

        groups = dict()
        unit_organization_lookup = 'scope__organization'
        unit_organization_field_name = Unit.get_organization_field_name()
        if CACHE['UNIT_MODEL'] and unit_organization_field_name:
            unit_organization_lookup = 'scope__unit__{}__{}'.format(
                CACHE['UNIT_MODEL'].__name__.lower(), unit_organization_field_name
            )
        scope_queryset = self.role_set.filter(
            active=True).values_list('group__name', 'scope__organization', 'scope__unit', unit_organization_lookup)
        for group_name, organization_id, unit_id, unit_organization_id in scope_queryset:

            if group_name not in groups:
                groups[group_name] = dict(username_lookups=[], organization_ids=[], unit_ids=[])
            if group_name in role_lookups:
                groups[group_name]['username_lookups'].append(role_lookups[group_name])
            if organization_id:
                groups[group_name]['organization_ids'].append(organization_id)
            if unit_id:
                groups[group_name]['unit_ids'].append(unit_id)
                if unit_organization_id:
                    groups[group_name]['organization_ids'].append(unit_organization_id)

        if model in CACHE['PERMISSIONS_BY_SCOPE']:
            can_view_globally = can_edit_globally = can_delete_globally = False
            for group_name in groups:

                username_lookups = groups[group_name]['username_lookups']
                unit_ids = list(set(groups[group_name]['unit_ids']))
                organization_ids = list(set(groups[group_name]['organization_ids']))

                can_view = can_view_by_role = can_view_by_unit = can_view_by_organization = False

                if obj:

                    if type(obj) in CACHE['PERMISSIONS_BY_SCOPE']:
                        can_view = group_name in CACHE['PERMISSIONS_BY_SCOPE'][type(obj)].get('add', [])
                        can_view_by_unit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][type(obj)].get('add_by_unit', [])
                        can_view_by_organization = group_name in CACHE['PERMISSIONS_BY_SCOPE'][type(obj)].get(
                            'add_by_organization', [])

                if (can_view or can_view_by_role or can_view_by_unit or can_view_by_organization) is False:
                    can_view = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('view', [])
                    can_view_by_role = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('view_by_role', [])
                    can_view_by_unit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('view_by_unit', [])
                    can_view_by_organization = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                        'view_by_organization', [])

                if can_view:
                    can_view_globally = True
                else:
                    if can_view_by_role:
                        for username_lookup in username_lookups:
                            lookups['list_lookups'].append((username_lookup, (self.username,)))
                    if can_view_by_unit:
                        if unit_ids:
                            for unit_lookup in unit_lookups:
                                lookups['list_lookups'].append(('{}'.format(unit_lookup), unit_ids))
                    if can_view_by_organization:
                        if organization_ids:
                            for organization_lookup in organization_lookups:
                                lookups['list_lookups'].append(('{}'.format(organization_lookup), organization_ids))

                can_edit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('edit', [])
                can_edit_by_role = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('edit_by_role', [])
                can_edit_by_unit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('edit_by_unit', [])
                can_edit_by_organization = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                    'edit_by_organization', [])

                if can_edit:
                    can_edit_globally = True
                else:
                    if can_edit_by_role:
                        for username_lookup in username_lookups:
                            lookups['edit_lookups'].append((username_lookup, (self.username,)))
                    if can_edit_by_unit and unit_ids:
                        for unit_lookup in unit_lookups:
                            lookups['edit_lookups'].append((unit_lookup, unit_ids))
                    if can_edit_by_organization and organization_ids:
                        for organization_lookup in organization_lookups:
                            lookups['edit_lookups'].append((organization_lookup, organization_ids))

                can_delete = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('delete', [])
                can_delete_by_role = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('delete_by_role', [])
                can_delete_by_unit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('delete_by_unit', [])
                can_delete_by_organization = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                    'delete_by_organization', [])

                if can_delete:
                    can_delete_globally = True
                else:
                    if can_delete_by_role:
                        for username_lookup in username_lookups:
                            lookups['delete_lookups'].append((username_lookup, (self.username,)))
                    if can_delete_by_unit and unit_ids:
                        for unit_lookup in unit_lookups:
                            lookups['delete_lookups'].append((unit_lookup, unit_ids))
                    if can_delete_by_organization and organization_ids:
                        for organization_lookup in organization_lookups:
                            lookups['delete_lookups'].append((organization_lookup, organization_ids))

                for actions_dict in (CACHE['INSTANCE_ACTIONS'], CACHE['QUERYSET_ACTIONS']):
                    for category in actions_dict.get(model, ()):
                        for key in list(actions_dict[model][category].keys()):
                            execute_lookups = []
                            view_name = actions_dict[model][category][key]['view_name']
                            can_execute = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get('{}'.format(
                                view_name), [])
                            can_execute_by_role = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                                '{}_by_role'.format(view_name), [])
                            can_execute_by_unit = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                                '{}_by_unit'.format(view_name), [])
                            can_execute_by_organization = group_name in CACHE['PERMISSIONS_BY_SCOPE'][model].get(
                                '{}_by_organization'.format(view_name), [])
                            if can_execute:
                                execute_lookups = None
                            else:
                                if can_execute_by_role:
                                    for username_lookup in username_lookups:
                                        execute_lookups.append((username_lookup, (self.username,)))
                                if can_execute_by_unit and unit_ids:
                                    for unit_lookup in unit_lookups:
                                        execute_lookups.append((unit_lookup, unit_ids))
                                if can_execute_by_organization and organization_ids:
                                    for organization_lookup in organization_lookups:
                                        execute_lookups.append((organization_lookup, organization_ids))
                            if execute_lookups:
                                if view_name not in lookups:
                                    lookups[view_name] = []
                                lookups[view_name] += execute_lookups

            if can_view_globally:
                lookups['list_lookups'] = []
            if can_edit_globally:
                lookups['edit_lookups'] = []
            if can_delete_globally:
                lookups['delete_lookups'] = []

        for codename in ('view', 'list'):
            if model in CACHE['PERMISSIONS_BY_SCOPE']:
                if CACHE['PERMISSIONS_BY_SCOPE'][model].get('{}_by_unit'.format(codename)) and not unit_lookups:
                    raise Exception('A "lookup" meta-attribute must point to a Unit model in {}'.format(model))
                if CACHE['PERMISSIONS_BY_SCOPE'][model].get('{}_by_organization'.format(codename)) and (
                        not organization_lookups and not unit_lookups):
                    raise Exception('A "lookup" meta-attribute must point to a Unit or Organization model in {}'.format(
                        model))
                if CACHE['PERMISSIONS_BY_SCOPE'][model].get('{}_by_role'.format(codename)) and not role_lookups:
                    raise Exception('A "lookup" meta-attribute must point to a role model in {}'.format(model))

        self.permission_mapping[permission_mapping_key] = lookups
        self.save()

        return self.permission_mapping[permission_mapping_key]

    # given a set of group or permission names, this method returns the groups the user belongs to
    def find_groups(self, perm_or_group, exclude=None):
        qs = self.groups.all()
        if perm_or_group:
            permissions = []
            groups = []
            for item in perm_or_group:
                if '.' in item:
                    permissions.append(item.split('.')[1])
                else:
                    groups.append(item)
            if permissions:
                qs = qs.filter(permissions__codename__in=permissions)
            if groups:
                qs = qs.filter(name__in=groups)

        if exclude:
            qs = qs.exclude(name=exclude)
        return qs

    def email_user(self, subject, message, from_email=None, **kwargs):
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def check_role_groups(self):
        for group in self.groups.all():
            self.groups.remove(group)
        for group in self.role_set.filter(active=True).values_list('group', flat=True).distinct():
            self.groups.add(group)


class Role(models.Model):
    user = models.ForeignKey('admin.User', verbose_name=_('User'), composition=True)
    group = models.ForeignKey('admin.Group', verbose_name=_('Group'))
    scope = models.ForeignKey('admin.Scope', verbose_name=_('Scope'), null=True, blank=True)
    active = models.BooleanField(verbose_name=_('Active'), default=True)
    fieldsets = (
        (_('General Data'), {'fields': ('user', 'group')}),
        (_('Scope'), {'fields': ('scope', 'active')}),
    )

    objects = models.Manager()

    class Meta:
        verbose_name = _('Role')
        verbose_name_plural = _('Roles')
        can_admin = _('User Manager')

    def save(self, *args, **kwargs):
        super(Role, self).save(*args, **kwargs)
        if self.active:
            self.user.groups.add(self.group)

    def __str__(self):
        return '{}'.format(self.group)


class PermissionManager(models.Manager):

    def get_queryset(self):
        app_labels = []
        for app_label in settings.INSTALLED_APPS:
            if '.' not in app_label:
                app_labels.append(app_label)
        return super(PermissionManager, self).get_queryset().filter(content_type__app_label__in=app_labels)

    # @action('Give to Users')
    def give_to_users(self):
        pass


class Permission(Permission):
    class Meta:
        verbose_name = _('Permission')
        verbose_name_plural = _('Permissions')
        proxy = True
        icon = 'fa-check'

    fieldsets = (
        (_('General Data'), {'fields': ('name',)}),
        (_('Users'), {'relations': ('user_set',)}),
    )

    objects = PermissionManager()


setattr(getattr(Permission, '_meta'), 'search_fields', ['codename', 'name'])
setattr(getattr(Permission, '_meta'), 'list_filter', ['content_type'])


class Group(Group):
    class Meta:
        verbose_name = _('Group')
        verbose_name_plural = _('Groups')
        proxy = True
        icon = 'fa-users'
        add_form = 'GroupForm'

    fieldsets = (
        (_('General Data'), {'fields': ('name',)}),
        ('{}::{}'.format(_('Permissions'), _('Permissions')), {'relations': ('permissions',)}),
        ('{}::{}'.format(_('Users'), _('Users')), {'relations': ('user_set',)}),
    )

    objects = models.Manager()


class Settings(models.Model):

    class Meta:
        verbose_name = _('Settings')
        verbose_name_plural = _('Settings')

    fieldsets = (
        (_('General Data'), {'fields': (('initials', 'name'), ('logo', 'logo_pdf'), ('icon', 'background'))}),
        (_('Social Data'), {'fields': (('twitter', 'facebook'), ('google', 'pinterest'), ('linkedin', 'rss'))}),
        (_('Copyright'), {'fields': ('company', ('phone_1', 'phone_2'), 'address', 'email')}),
        (_('Look and Feel'), {'fields': ('default_color',)}),
        (_('Server'), {'fields': (('server_address', 'system_email_address'),)}),
        (_('Version'), {'fields': ('version',)}),
    )

    # Application
    initials = models.CharField(_('Name'), default='Django+')
    name = models.CharField(_('Description'), default='Django Plus')
    logo = models.ImageField(_('Logo'), upload_to='config', null=True, blank=True, default='')
    logo_pdf = models.ImageField(_('PDF Logo'), upload_to='config', help_text=_('No-background image'),
                                 null=True, blank=True, default='')
    icon = models.ImageField(_('Icon'), upload_to='config', null=True, blank=True)
    background = models.ImageField('Background', upload_to='config', default='', blank=True)

    # Social params
    twitter = models.CharField('Twitter', null=True, blank=True)
    facebook = models.CharField('Facebook', null=True, blank=True)
    google = models.CharField('Google', null=True, blank=True)
    pinterest = models.CharField('Pinterest', null=True, blank=True)
    linkedin = models.CharField('Linkedin', null=True, blank=True)
    rss = models.CharField('RSS', null=True, blank=True)

    # Company
    company = models.CharField(_('Company Name'), null=True, blank=True)
    address = models.TextField(_('Address'), null=True, blank=True)
    phone_1 = models.PhoneField(_('Primary Phome'), null=True, blank=True)
    phone_2 = models.PhoneField(_('Secondary Phone'), null=True, blank=True)
    email = models.CharField(_('Email'), null=True, blank=True)

    # Server configuration
    version = models.CharField(_('System Version'), exclude=True)
    server_address = models.CharField(_('Server URL'), default='http://localhost:8000')
    system_email_address = models.CharField(_('Notificaton E-mail'), default='no-reply@djangoplus.net')

    @staticmethod
    def default():
        from djangoplus.cache import CACHE
        if not CACHE['SETTINGS_INSTANCE']:
            CACHE['SETTINGS_INSTANCE'] = Settings.objects.first()
        if not CACHE['SETTINGS_INSTANCE']:
            s = Settings()
            s.initials = _('System')
            s.name = _('Online, responsive e multiplatform system')
            s.twitter = 'https://twitter.com/'
            s.facebook = 'https://www.facebook.com/'
            s.google = 'https://plus.google.com/'
            s.pinterest = 'https://www.pinterest.com/'
            s.linkedin = 'https://www.linkedin.com/'
            s.rss = 'https://www.rss.com/'
            s.company = ''
            s.address = ''
            s.phone_1 = ''
            s.phone_2 = ''
            s.email = ''
            s.version = '1.0'
            s.save()
            CACHE['SETTINGS_INSTANCE'] = s
        return CACHE['SETTINGS_INSTANCE']

    def save(self, *args, **kwargs):
        from djangoplus.cache import CACHE
        super(Settings, self).save(*args, **kwargs)
        CACHE['SETTINGS_INSTANCE'] = self



# -*- coding: utf-8 -*-
import os
import sys
import time
import random
from subprocess import Popen, PIPE
from django.utils import termcolors

TYPING_SPEED = 50


def simulate_command_type(commands, shell=False):
    for command in commands.split('&& '):
        if not command.startswith('source'):
            simulate_type(command, shell=shell)


def simulate_type(command, shell=False):
    sys.stdout.write('breno@localhost: ~$ ')
    for c in command:
        sys.stdout.write(c)
        sys.stdout.flush()
        time.sleep(random.random()*10.0/TYPING_SPEED)
    print('')


def bold(text):
    return termcolors.make_style(fg='black', opts=('bold',))(text)


def info(text):
    return termcolors.make_style(fg='cyan')(text)


def error(text):
    return termcolors.make_style(fg='red', opts=('bold',))(text)


class Terminal(object):

    def __init__(self, verbose=True, python='python'):
        self.proccess = None
        self.verbose = verbose
        self.python = python

    def execute(self, command, clear=True, base_dir=None):
        if clear:
            os.system('clear')
            simulate_type('', shell=True)
        if self.verbose:
            simulate_command_type(command, shell=True)
        if command.startswith('python'):
            command.replace('python', self.python)
        if base_dir:
            command = 'cd {} && {}'.format(base_dir, command)
        if not self.verbose:
            command = '{}  > /dev/null'.format(command)
        os.system(command)

    def show(self, visible=True):
        if os.path.exists('/usr/bin/osascript'):
            minimize_terminal_script = '''
                tell application "Terminal"
                  set miniaturized of window 1 to {}
                end tell
            '''.format(visible and 'false' or 'true')
            self.proccess = Popen(['osascript', '-'], stdin=PIPE, stdout=PIPE, stderr=PIPE)
            self.proccess.communicate(minimize_terminal_script.encode())

    def hide(self):
        self.show(False)




# -*- coding: utf-8 -*-
import os
from django.core.management.base import BaseCommand
from fabric.api import *

env.user = 'root'


class Command(BaseCommand):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--update', action='store_true', dest='update', default=False,
                            help='Updates djangoplus.net')
        parser.add_argument('--update-samples', action='store_true', dest='update_samples', default=False,
                            help='Updates samples at djangoplus.net')
        parser.add_argument('--test-samples', action='store_true', dest='test_samples', default=False,
                            help='Tests samples at djangoplus.net')

    def handle(self, *args, **options):
        if options.get('update'):
            execute(_update_site, host='djangoplus.net')
        if options.get('update_samples'):
            execute(_update_samples, host='djangoplus.net')
        if options.get('test_samples'):
            execute(_test_samples, host='djangoplus.net')


def _update_site():
    if os.path.exists('/Users/breno'):
        with lcd('/Users/breno/Documents/Workspace/djangoplus/site'):
            if 'nothing to commit' not in local('git status', capture=True):
                local('git commit -am \'.\'')
                local('git push origin master')
        with cd('/var/www/html'):
            run('git pull origin master')


def _test_samples():
    with lcd('/tmp'):
        for project_name in ('petshop', 'loja', 'biblioteca'):
            project_path = '/tmp/{}'.format(project_name)
            zip_path = '/{}.zip'.format(project_name)
            if os.path.exists(zip_path):
                local('rm {}'.format(zip_path))
            if os.path.exists(project_path):
                local('rm -r {}'.format(project_path))
            local('wget http://{}.djangoplus.net/media/{}.zip'.format(project_name, project_name))
            local('unzip {}.zip -d {}'.format(project_name, project_name))
            with lcd(project_path):
                local('/Users/breno/Envs/djangoplus/bin/python manage.py test')


def _update_samples():
    for project_name in ('petshop', 'loja', 'biblioteca'):
        if os.path.exists('/Users/breno'):
            project_path = '/Users/breno/Documents/Workspace/djangoplus/djangoplus-demos/{}'.format(project_name)
            with lcd(project_path):
                local('pwd')
                if 'nothing to commit' not in local('git status', capture=True):
                    local('git commit -am \'.\'')
                    local('git push origin master')
            with cd('/var/opt/{}'.format(project_name)):
                run('pwd')
                run('git checkout .')
                run('git pull origin master')
                if project_name == 'petshop':
                    run("sed -i 's/#:.*//g' petshop/models.py")
                run('workon {} && python manage.py zip'.format(project_name))


import os
from subprocess import check_call
from setuptools import find_packages, setup
from setuptools.command.install import install


# with open(os.path.join(os.path.dirname(__file__), 'djangoplus/README.md')) as readme:
#    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


class PostInstallCommand(install):

    def run(self):
        check_call("/bin/bash djangoplus/bin/setpath".split())
        install.run(self)


setup(
    name='djangoplus',
    version='0.0.93',
    packages=find_packages(),
    install_requires=[
        'Django==2.1.7', 'pdfkit==0.6.1', 'python-dateutil==2.8.0', 'selenium==3.141.0', 'xlwt==1.3.0', 'xlrd==1.2.0', 'unicodecsv==0.14.1', 'qrcode==6.1', 'requests==2.21.0', 'django-jinja==2.4.1', 'Pillow==6.0.0', 'gunicorn==19.9.0', 'Fabric3==1.14.post1', 'google-auth-oauthlib==0.3.0', 'google-api-python-client==1.7.8', 'oauth2client==4.1.3', 'dropbox==9.3.0', 'Fabric3==1.14.post1'
    ],
    extras_require={
        'dev': [],
        'production': [],
    },
    scripts=[
        'djangoplus/bin/startproject', 'djangoplus/bin/install-djangoplus-tools','djangoplus/bin/test-djangoplus-tools', 'djangoplus/bin/setpath'
    ],
    include_package_data=True,
    license='BSD License',
    description='Metadata-based web framework for the development of management information systems',
    long_description='',  # README,
    cmdclass={
        'install': PostInstallCommand,
    },
    url='http://djangoplus.net/',
    author='Breno Silva',
    author_email='brenokcc@yahoo.com.br',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# Generated by Django 2.0.8 on 2019-02-12 15:19

from django.db import migrations


def migrate(apps, schema):
    for model in apps.get_models():
        for field in model._meta.fields:
            if field.__class__.__name__ == 'ImageField':
                default = field.default
                if default and type(default) in (bytes, str):
                    if type(default) == bytes:
                        default = default.decode()
                        model.objects.filter(**{'{}__contains'.format(field.name): '/static/'}).update(
                            **{field.name: default.split('/')[-1]})


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0007_auto_20180416_1854'),
    ]

    operations = [migrations.RunPython(migrate)]



# -*- coding: utf-8 -*-

from djangoplus.utils.metadata import iterable
from djangoplus.utils.http import return_response
from django.utils.translation import ugettext as _
from django.http.response import HttpResponseRedirect
from djangoplus.utils import permissions, get_metadata


def view(verbose_name, icon=None, menu=None, login_required=True, style='ajax', template=None, shortcut=False,
         usecase=None, can_view=()):

    def decorate(func):
        url = '/{}/{}/'.format(func.__module__.split('.')[-2], func.__name__)

        def receive_function_args(request, *args, **kwargs):
            without_permission = can_view and not permissions.check_group_or_permission(request, can_view)
            without_authentication = login_required and not request.user.is_authenticated
            if without_permission or without_authentication:
                    return HttpResponseRedirect('/admin/login/?next={}'.format(url))
            f_return = func(request, *args, **kwargs)
            template_name = template or '{}.html'.format(func.__name__)
            return return_response(f_return, request, verbose_name, style, template_name)

        receive_function_args._view = dict(
            verbose_name=verbose_name, function=func, url=url, can_view=iterable(can_view), menu=menu, icon=icon,
            style=style, add_shortcut=shortcut, doc=func.__doc__, usecase=usecase
        )
        return receive_function_args

    return decorate


def action(model, verbose_name, help_text=None, condition=None, inline=False, subset=(), icon=None, category=None,
           style='ajax', message=_('Action successfully performed'), menu=None, can_execute=(),
           can_execute_by_organization=None, can_execute_by_unit=None, can_execute_by_role=None, usecase=None):

    def decorate(func):
        def receive_function_args(request, *args, **kwargs):
            if can_execute and not permissions.check_group_or_permission(
                    request, '{}.{}'.format(get_metadata(model, 'app_label'), func.__name__)):
                return HttpResponseRedirect('/admin/login/')
            f_return = func(request, *args, **kwargs)
            template_name = '{}.html'.format(func.__name__)
            return return_response(f_return, request, verbose_name, style, template_name)

        _action = dict(
            verbose_name=verbose_name, can_execute=iterable(can_execute), condition=condition, help_text=help_text,
            view_name=func.__name__, function=func, group=category, style=style, message=message, model=model,
            input=None, initial=None, choices=None, inline=inline, subsets=iterable(subset), icon=icon,
            doc=func.__doc__, usecase=usecase, can_execute_by_organization=iterable(can_execute_by_organization),
            can_execute_by_unit=iterable(can_execute_by_unit), can_execute_by_role=iterable(can_execute_by_role),
            redirect_to=None, menu=menu, display=None, source='view', expose=('web',)
        )

        receive_function_args._action = _action
        func._view_action = _action

        return receive_function_args

    return decorate


def dashboard(can_view=(), position='bottom'):
    def decorate(func):
        def receive_function_args(request, *args, **kwargs):
            return func(request, *args, **kwargs)
        receive_function_args._widget = dict(function=func, can_view=can_view, position=position)
        return receive_function_args
    return decorate



# Generated by Django 2.0.3 on 2018-04-16 10:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('admin', '0003_auto_20180416_1034'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='organization',
            name='ascii',
        ),
        migrations.RemoveField(
            model_name='unit',
            name='ascii',
        ),
        migrations.RenameField(
            model_name='organization',
            old_name='id',
            new_name='scope_ptr',
        ),
        migrations.RenameField(
            model_name='unit',
            old_name='id',
            new_name='scope_ptr',
        ),
        migrations.AlterField(
            model_name='organization',
            name='scope_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='admin.Scope'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='scope_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='admin.Scope'),
        ),
    ]



# -*- coding: utf-8 -*-
import os
import json
import warnings
from django.conf import settings
from djangoplus.tools.browser import Browser
from djangoplus.tools.subtitle import Subtitle
from djangoplus.tools.terminal import Terminal
from django.core.management import call_command
from djangoplus.tools.video import VideoRecorder
from django.utils.translation import ugettext_lazy as _
from django.contrib.staticfiles.testing import LiveServerTestCase
from django.contrib.staticfiles.handlers import StaticFilesHandler


CACHE = dict(
    SEQUENCE=0,
    STEP=None,
    CONTINUE=None,
    RESUME=None,
    RECORD=None,
    HEADLESS=None,
    PAUSE=False
)


class TestStaticFilesHandler(StaticFilesHandler):
    def _middleware_chain(self, request):
        from django.http import HttpResponse
        return HttpResponse()


# StaticLiveServerTestCase
class TestCase(LiveServerTestCase):

    static_handler = TestStaticFilesHandler
    AUDIO_FILE_PATH = None

    def __init__(self, *args, **kwargs):
        super(TestCase, self).__init__(*args, **kwargs)
        self.login_count = 0
        self.current_username = None
        warnings.filterwarnings('ignore')

    @classmethod
    def setUpClass(cls):
        super(TestCase, cls).setUpClass()
        cls.username = None
        cls.browser = Browser(cls.live_server_url)
        cls.subtitle = Subtitle()
        cls.recorder = VideoRecorder()
        cls.terminal = Terminal()
        if CACHE['RECORD'] and not CACHE['HEADLESS']:
            cls.terminal.hide()
        if CACHE['RECORD']:
            cls.browser.slowly = True
        for app_label in settings.INSTALLED_APPS:
            app_module = __import__(app_label)
            app_dir = os.path.dirname(app_module.__file__)
            fixture_path = os.path.join(app_dir, 'fixtures', 'test.json')
            if os.path.exists(fixture_path):
                call_command('loaddata', fixture_path)

    def create_superuser(self, username, password):
        from djangoplus.admin.models import User
        if not User.objects.filter(username=username).exists():
            User.objects.create_superuser(username, None, password)
        self.wait(1)

    def wait(self, seconds=1):
        self.browser.wait(seconds)

    def open(self, url):
        self.browser.open(url)

    def back(self, seconds=None):
        self.browser.back(seconds)

    def enter(self, name, value, submit=False, count=2):
        self.browser.enter(name, value, submit, count)

    def choose(self, name, value, count=2):
        self.browser.choose(name, value, count)

    def check(self, name=None):
        self.browser.check(name)

    def dont_see_error_message(self):
        self.browser.dont_see_error_message(self)

    def see(self, text, flag=True, count=2):
        self.browser.see(text, flag, count)

    def look_at_popup_window(self, count=2):
        self.browser.look_at_popup_window(count)

    def look_at(self, text, count=2):
        self.browser.look_at(text, count)

    def look_at_panel(self, text, count=2):
        self.browser.look_at_panel(text, count)

    def click_menu(self, *texts):
        self.browser.click_menu(*texts)

    def click_link(self, text):
        self.browser.click_link(text)

    def click_button(self, text):
        self.browser.click_button(text)

    def click_tab(self, text):
        self.browser.click_tab(text)

    def click_icon(self, name):
        self.browser.click_icon(name)

    def login(self, username, password):
        self.current_username = username
        self.open('/admin/login/')
        self.enter(_('Username'), username)
        self.enter(_('Password'), password)
        self.click_button(_('Sign-in'))
        self.wait()

    def logout(self):
        self.browser.logout()
        self.username = None

    @classmethod
    def tearDownClass(cls):
        super(TestCase, cls).tearDownClass()
        if CACHE['RECORD'] and not CACHE['HEADLESS']:
            cls.terminal.show()
        cls.browser.close()
        cls.browser.service.stop()
        cls.recorder.stop()
        # len(self._resultForDoCleanups.errors)>0

    def dump(self, step):
        from djangoplus.admin.models import User
        file_path = '/tmp/{}.test'.format(settings.PROJECT_NAME)
        dump_file_path = '/tmp/{}_{}.json'.format(settings.PROJECT_NAME, step)
        data = dict(step=step)
        open(file_path, 'w').write(json.dumps(data))
        output = open(dump_file_path,'w')
        app_labels = []
        for app in settings.INSTALLED_APPS:
            app_label = app.split('.')[-1]
            if app_label not in 'auth':
                app_labels.append(app_label)
        User.objects.update(permission_mapping=None)
        call_command('dumpdata', *app_labels, format='json', indent=3, stdout=output, skip_checks=True, verbosity=0)
        output.close()

    def restore(self, step):
        dump_file_path = '/tmp/{}_{}.json'.format(settings.PROJECT_NAME, step)
        call_command('loaddata', dump_file_path)

    def execute_flow(self):
        flow = []

        for attr_name in dir(self):
            if not attr_name.startswith('_'):
                attr = getattr(self, attr_name)
                if hasattr(attr, '_sequence'):
                    flow.append(attr)

        if CACHE['STEP'] or CACHE['RESUME'] or CACHE['CONTINUE']:
            execute = False
        else:
            execute = True
        flow.sort(key=lambda x: x._sequence)
        for i, testcase in enumerate(flow):
            if execute:
                testcase()
            elif CACHE['RESUME']:
                if CACHE['RESUME'] == testcase._funcname:
                    self.restore(CACHE['RESUME'])
                    execute = True
            elif CACHE['STEP']:
                if CACHE['STEP'] == testcase._funcname:
                    if i > 0:
                        self.restore(flow[i-1]._funcname)
                    testcase()
            elif CACHE['CONTINUE']:
                if CACHE['CONTINUE'] == testcase._funcname:
                    if i > 0:
                        self.restore(flow[i-1]._funcname)
                    testcase()
                    execute = True

        if CACHE['PAUSE'] and not CACHE['HEADLESS']:
            input('Type enter to continue...')



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-03-23 21:18


from django.conf import settings
import django.contrib.auth.models
from django.db import migrations, models
import django.db.models.deletion
import djangoplus.admin.models
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    replaces = [('admin', '0001_initial'), ('admin', '0002_log_logindex'), ('admin', '0003_auto_20160914_1821'), ('admin', '0004_auto_20161001_1750'), ('admin', '0005_auto_20161005_1536'), ('admin', '0006_organization_organizationrole'), ('admin', '0007_auto_20161124_1811'), ('admin', '0008_auto_20161124_1825'), ('admin', '0009_auto_20170420_1238'), ('admin', '0010_auto_20170428_1545'), ('admin', '0011_user_permission_mapping'), ('admin', '0012_auto_20170619_1648'), ('admin', '0013_auto_20170910_1543'), ('admin', '0014_auto_20180106_1921'), ('admin', '0015_settings_company')]

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'managed': False,
                'db_table': 'admin_user_groups',
                'verbose_name': 'Fun\xe7\xe3o',
                'verbose_name_plural': 'Fun\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('name', djangoplus.db.models.fields.CharField(blank=True, max_length=30, verbose_name='Nome')),
                ('username', djangoplus.db.models.fields.CharField(max_length=30, unique=True, verbose_name='Login')),
                ('email', djangoplus.db.models.fields.CharField(blank=True, default='', max_length=75, verbose_name='E-mail')),
                ('active', djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Ativo?')),
                ('photo', djangoplus.db.models.fields.ImageField(blank=True, default='user.png', null=True, upload_to='profiles', verbose_name='Foto')),
            ],
            options={
                'verbose_name': 'Usu\xe1rio',
                'verbose_name_plural': 'Usu\xe1rios',
            },
            managers=[
                ('objects', djangoplus.admin.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Settings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('initials', djangoplus.db.models.fields.CharField(default='Django+', max_length=255, verbose_name='Nome')),
                ('name', djangoplus.db.models.fields.CharField(default='Django Plus', max_length=255, verbose_name='Descri\xe7\xe3o')),
                ('logo', djangoplus.db.models.fields.ImageField(blank=True, default='', null=True, upload_to='config', verbose_name='Logotipo')),
                ('logo_pdf', djangoplus.db.models.fields.ImageField(blank=True, default='', help_text='Imagem sem fundo transparente', null=True, upload_to='config', verbose_name='Logotipo para PDF')),
                ('icon', djangoplus.db.models.fields.ImageField(blank=True, default='blank.png', null=True, upload_to='config', verbose_name='\xcdcone')),
                ('background', djangoplus.db.models.fields.ImageField(blank=True, default='', upload_to='config', verbose_name='Background')),
                ('twitter', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Twitter')),
                ('facebook', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Facebook')),
                ('google', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Google')),
                ('pinterest', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='pinterest')),
                ('linkedin', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Linkedin')),
                ('rss', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='RSS')),
                ('address', djangoplus.db.models.fields.TextField(blank=True, null=True, verbose_name='Endere\xe7o')),
                ('phone_1', djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Principal')),
                ('phone_2', djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Secund\xe1rio')),
                ('email', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='E-mail')),
                ('version', djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='Vers\xe3o do Sistema')),
                ('server_address', djangoplus.db.models.fields.CharField(default='http://localhost:8000', max_length=255, verbose_name='Endere\xe7o de Acesso')),
                ('system_email_address', djangoplus.db.models.fields.CharField(default='no-reply@djangoplus.net', max_length=255, verbose_name='E-mail de Notifica\xe7\xe3o')),
                ('company', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Empresa')),
            ],
            options={
                'verbose_name': 'Configura\xe7\xe3o',
                'verbose_name_plural': 'Configura\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='Unit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ascii', djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False)),
            ],
            options={
                'verbose_name': 'Unidade',
                'verbose_name_plural': 'Unidades',
            },
        ),
        migrations.CreateModel(
            name='UnitRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Role', verbose_name='Fun\xe7\xe3o')),
                ('unit', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Unit', verbose_name='Unidade')),
            ],
            options={
                'verbose_name': 'Fun\xe7\xe3o na Unidade',
                'verbose_name_plural': 'Fun\xe7\xf5es na Unidade',
            },
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
            ],
            options={
                'verbose_name': 'Grupo',
                'proxy': True,
                'verbose_name_plural': 'Grupos',
            },
            bases=('auth.group',),
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.AddField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='user',
            name='user_permissions',
            field=models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions'),
        ),
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('operation', djangoplus.db.models.fields.IntegerField(choices=[[1, 'Cadastro'], [2, 'Edi\xe7\xe3o'], [3, 'Exclus\xe3o']], verbose_name='Opera\xe7\xe3o')),
                ('date', djangoplus.db.models.fields.DateTimeField(auto_now=True, verbose_name='Data/Hora')),
                ('object_id', djangoplus.db.models.fields.IntegerField(verbose_name='Identificador')),
                ('object_description', djangoplus.db.models.fields.CharField(max_length=255, verbose_name='Descri\xe7\xe3o do Objeto')),
                ('content', djangoplus.db.models.fields.TextField(null=True, verbose_name='Conte\xfado')),
                ('content_type', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Objeto')),
                ('user', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
            },
        ),
        migrations.CreateModel(
            name='LogIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', djangoplus.db.models.fields.IntegerField(verbose_name='Identificador')),
                ('content_type', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Dado')),
                ('log', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Log', verbose_name='Log')),
            ],
            options={
                'verbose_name': 'Index',
                'verbose_name_plural': 'Indexes',
            },
        ),
        migrations.CreateModel(
            name='ContentType',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('contenttypes.contenttype',),
            managers=[
                ('objects', djangoplus.admin.models.ContentTypeManager()),
            ],
        ),
        migrations.AlterField(
            model_name='log',
            name='content_type',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.ContentType', verbose_name='Objeto'),
        ),
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ascii', djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False)),
            ],
            options={
                'verbose_name': 'Organiza\xe7\xe3o',
                'verbose_name_plural': 'Organiza\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='OrganizationRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('organization', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Organization', verbose_name='Organiza\xe7\xe3o')),
                ('role', djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Role', verbose_name='Fun\xe7\xe3o')),
            ],
            options={
                'verbose_name': 'Papel na Organiza\xe7\xe3o',
                'verbose_name_plural': 'Papeis na Organiza\xe7\xe3o',
            },
        ),
        migrations.AlterField(
            model_name='user',
            name='is_superuser',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Superusu\xe1rio?'),
        ),
        migrations.AlterField(
            model_name='user',
            name='password',
            field=djangoplus.db.models.fields.CharField(max_length=128, verbose_name='Senha'),
        ),
        migrations.AlterField(
            model_name='user',
            name='is_superuser',
            field=models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status'),
        ),
        migrations.AlterField(
            model_name='user',
            name='password',
            field=models.CharField(max_length=128, verbose_name='password'),
        ),
        migrations.CreateModel(
            name='Permission',
            fields=[
            ],
            options={
                'verbose_name': 'Permiss\xe3o',
                'proxy': True,
                'verbose_name_plural': 'Permiss\xf5es',
                'indexes': [],
            },
            bases=('auth.permission',),
        ),
        migrations.AddField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.JsonField(null=True, verbose_name='Mapeamento de Permiss\xe3o'),
        ),
        migrations.AddField(
            model_name='user',
            name='organization',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Organization', verbose_name='Organiza\xe7\xe3o'),
        ),
        migrations.AddField(
            model_name='user',
            name='unit',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Unit', verbose_name='Unidade'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='unit',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='organization',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False),
        ),
        migrations.AlterField(
            model_name='unit',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default='', editable=False),
        ),
    ]



# Generated by Django 2.0.3 on 2018-04-16 10:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0004_auto_20180416_1037'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='organizationrole',
            name='organization',
        ),
        migrations.RemoveField(
            model_name='organizationrole',
            name='role',
        ),
        migrations.RemoveField(
            model_name='unitrole',
            name='role',
        ),
        migrations.RemoveField(
            model_name='unitrole',
            name='unit',
        ),
        migrations.DeleteModel(
            name='OrganizationRole',
        ),
        migrations.DeleteModel(
            name='UnitRole',
        ),
        migrations.DeleteModel(
            name='Role',
        ),
    ]



# -*- coding: utf-8 -*-

import os
import zipfile
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        zip_file_name = '{}.zip'.format(settings.PROJECT_NAME)

        def zipdir(file_path, ziph):
            for dirname, subdirs, files in os.walk(file_path):
                for filename in files:
                    absname = os.path.abspath(os.path.join(dirname, filename))
                    arcname = absname[len(settings.BASE_DIR) + 1:]
                    ignore = ('.git', 'gunicorn_start.sh', zip_file_name, 'logs', 'static')
                    add_file = True
                    for word in ignore:
                        if word in arcname:
                            add_file = False
                    if add_file:
                        ziph.write(absname, arcname)

        zip_file_path = os.path.join(settings.MEDIA_ROOT, zip_file_name)
        zipf = zipfile.ZipFile(zip_file_path, 'w', zipfile.ZIP_DEFLATED)
        zipdir(settings.BASE_DIR, zipf)
        zipf.close()
